@@ -27,26 +27,30 @@ class TaskFailed(Exception):
     """
     Wrap the error's *reason* and *details* for an task that failed.
 
+    :param name: of the task that failed.
+    :type name: str.
     :param reason: of the failure.
     :type  reason: str.
     :param details: of the failure.
     :type  details: str.
 
     """
-    def __init__(self, reason, details=None):
-        super(TaskFailed, self).__init__(self, reason, details)
+    def __init__(self, name, reason, details=None):
+        super(TaskFailed, self).__init__(name, reason, details)
+        self.name = name
         self.reason = reason
         self.details = None
 
     def __repr__(self):
-        return '{}(reason="{}")'.format(
+        return '{} ({}, "{}")'.format(
             self.__class__.__name__,
+            self.name,
             self.reason,
-            self.details)
+        )
 
 
 class TimeoutError(Exception):
-    def __init__(self, timeout_type, timeout_value=None):
+    def __init__(self, timeout_type='unknown timeout', timeout_value=None):
         self.timeout_type = timeout_type
         self.timeout_value = timeout_value
 
