@@ -1,9 +1,11 @@
 import logging
 
 from simpleflow import (
+    exceptions,
     executor,
     futures,
 )
+from ..task import ActivityTask
 
 
 logger = logging.getLogger(__name__)
@@ -17,16 +19,17 @@ class Executor(executor.Executor):
     def submit(self, func, *args, **kwargs):
         logger.info('executing task {}(args={}, kwargs={})'.format(
             func, args, kwargs))
-        args = [executor.get_actual_value(arg) for arg in args]
-        kwargs = {key: executor.get_actual_value(val) for
-                  key, val in kwargs.iteritems()}
 
         future = futures.Future()
+
+        task = ActivityTask(func, *args, **kwargs)
+
         try:
-            future._result = func._callable(*args, **kwargs)
+            future._result = task.execute()
         except Exception as err:
             future._exception = err
-            raise
+            if func.raises_on_failure:
+                raise exceptions.TaskFailed(func.name, err.message)
         finally:
             future._state = futures.FINISHED
 
@@ -38,4 +41,9 @@ class Executor(executor.Executor):
         args = input.get('args', ())
         kwargs = input.get('kwargs', {})
 
-        return self.run_workflow(*args, **kwargs)
+        self.before_replay()
+        result = self.run_workflow(*args, **kwargs)
+        self.after_replay()
+        self.on_completed()
+        self.after_closed()
+        return result
